% simulationResults_fig4.m
%
% Generates simulation results presented in Fig. 4.
%
% Simulation of the variance iteration (22) for Example 1 and different
% values of the sending threshold \delta.
%
% --
% Supplementary material to
%
% S. Trimpe, R. D'Andrea, "Event-Based State Estimation with Variance-Based 
% Triggering", Proc. of the 51st IEEE Conference on Decision and Control, 
% 2012.
%
% Copyright:
% Sebastian Trimpe
% ETH Zurich
% Institute for Dynamic Systems and Control (IDSC)
% strimpe(at)ethz.ch
% March 2012
%

% Revision history
% [March 12, Seb]   first version

close all;


%% Configuration
% Uncomment to use latex interpreter.
set(0,'defaulttextinterpreter','latex');    % set text interpreter

% Line color in plots.
lineCol = 'b';


%% parameters
% Problem parameters: [a c delta]. Q and R are assumed 1 w.l.o.g.
param(1,:) = [1.2 1 0.2];       % 5-periodic solution (1-1-0-1-0)
param(2,:) = [1.2 1 3];         % 3-periodic solution (1-0-0)
param(3,:) = [1.2 1 9.6167];    % 19-periodic

% periods
N = [5 3 19];

% time horizon for simulation
K = 5*N;            % 5 times period length


%% Perform simulation

% Variable assignments 
nEx = size(param,1);    % number of examples

for i=1:nEx
    pp{i} = zeros(K(i)+1,1);        % vector to hold result for p(k)
    gamma{i} = zeros(K(i),1);       % gamma(p(k))
    kk{i} = 0:1:K(i);               % iteration indix k
end;

% Assign initial conditions: pbar; compute sending threshold
for i=1:nEx
    a = param(i,1);
    c = param(i,2);
    d = param(i,3);
    
    % Start iteration at the solution of the DARE.
    pp{i}(1) = dare(a,c,1,1);
    
    % Compute sending threshold (used only for illustration in figures).
    p2(i) = pp{i}(1) + d;
end;

% Simulate each example
for i=1:nEx
    a = param(i,1);
    c = param(i,2);
    d = param(i,3);
    
    for k=1:K(i)
        % Iterate: p(k+1) = h(p(k)), equation (22).
        % Also returns \gamma(k).
        [pp{i}(k+1),gamma{i}(k)] = function_h(pp{i}(k),a,c,d);
    end;
end;


%% Plot1: 5-periodic solution
fig1 = figure;
iEx = 1;        % example number
set(gcf,'Name',['(b) delta = ',num2str(param(iEx,3))]);

% pk iterates
subplot(3,1,1:2);
plot(kk{iEx},pp{iEx}(:),[lineCol,'.-'],[kk{iEx}(1),kk{iEx}(end)],[p2(iEx) p2(iEx)],'r');

ylabel('$p(k)$');
grid;
set(gca,'xlim',[kk{i}(1),kk{iEx}(end)]);
set(gca,'ylim',[min(pp{iEx}(:))-0.5,max(pp{iEx}(:))+0.5]);
set(gca,'XTick',[0:N(1):kk{iEx}(end)]);
set(gca,'XTickLabel','');
set(gca,'YTick',round(100*[pp{iEx}(26) pp{iEx}(24)])/100);
set(gca,'YTickLabel',{num2str(round(100*pp{iEx}(26))/100),num2str(round(100*pp{iEx}(24))/100)});

% transmit decisions gamma
subplot(3,1,3);
stem(kk{iEx}(1:end-1),gamma{iEx}(:),'.');

xlabel('$k$');
ylabel('$\gamma(p(k))$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[-0.2 1.2]);
set(gca,'XTick',[0:N(1):kk{iEx}(end)]);
set(gca,'YTick',[0 1]);


%% Plot2: 3-periodic solution
fig2 = figure;
iEx = 2;        % example number
set(gcf,'Name',['(a) delta = ',num2str(param(iEx,3))]);

% pk iterates
subplot(3,1,1:2);
plot(kk{iEx},pp{iEx}(:),[lineCol,'.-'],[kk{iEx}(1),kk{iEx}(end)],[p2(iEx) p2(iEx)],'r');

ylabel('$p(k)$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[min(pp{iEx}(:))-1,max(pp{iEx}(:))+1]);
set(gca,'XTick',[0:N(2):kk{iEx}(end)]);
set(gca,'XTickLabel','');
set(gca,'YTick',round( 100*[sort([pp{iEx}(end) pp{iEx}(end-1) pp{iEx}(end-2)]) ] )/100);

% transmit decisions gamma
subplot(3,1,3);
stem(kk{iEx}(1:end-1),gamma{iEx}(:),'.');

xlabel('$k$');
ylabel('$\gamma(p(k))$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[-0.2 1.2]);
set(gca,'XTick',[0:N(2):kk{iEx}(end)]);
set(gca,'YTick',[0 1]);


%% Plot3: 19-periodic solution
fig3 = figure;
iEx = 3;        % example number
set(gcf,'Name',['(c) delta = ',num2str(param(iEx,3))]);

% pk iterates
subplot(3,1,1:2);
plot(kk{iEx},pp{iEx}(:),[lineCol,'.-'],[kk{iEx}(1),kk{iEx}(end)],[p2(iEx) p2(iEx)],'r');

ylabel('$p(k)$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[min(pp{iEx}(:))-1,max(pp{iEx}(:))+1]);
set(gca,'XTick',[0:N(3):kk{iEx}(end)]);
set(gca,'XTickLabel','');
set(gca,'YTick',round(100*[sort(pp{iEx}(49:53)')])/100);

% transmit decisions gamma
subplot(3,1,3);
stem(kk{iEx}(1:end-1),gamma{iEx}(:),'.');

xlabel('$k$');
ylabel('$\gamma(p(k))$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[-0.2 1.2]);
set(gca,'XTick',[0:N(3):kk{iEx}(end)]);
set(gca,'YTick',[0 1]);

